/*
Desenvolvedores: Evandro Mendonça - evandro.user@gmail.com
                 Mateus Pelegrino - mateuspelegrino@gmail.com
Colaboração: Esteban Clua, Marcelo Zamith, Calebe Tonassi, Elaine Cifoni.
*/

/*COMENTÁRIO DA CLASSE
    Esta classe é responsável pelo Boss do jogo, o último inimigo do jogo. A classe
 gerência tudo que está relacionado ao Boss, como movimento, vida e sprites por exemplo.
*/

package aliensfront;

import java.awt.Color;
import java.awt.Graphics;
/*Importações de classes do JavaPlay, a primeira é usada para todo objeto do jogo,
 como personagens ou qualquer coisa que tenha um papel interativo no jogo. A segunda
 é usada para criar um Sprite, ou seja, um conjunto de desenhos que compõe o
 personagem ou objeto.*/
import javaPlay.GameObject;
import javaPlay.Sprite;

public class Boss extends GameObject {

    private Sprite boss;
    private Sprite hp;
    private int FrameAtual;
    private int FrameAtualHp;
    private int bossEixo;
    /*Verifica se o Boss tem a permisão para andar livremente pela tela.*/
    private boolean andarLivre = true;
    /*É o quantidade de vida que o Boss tem, cada ataque acertado corresponde a uma vida.*/
    private int vidas = 6;
    /*Variável usada para dar sensação de movimento no Boss.*/
    private int i = 0;
    /*Declara a posição de orientação dos sprites, ou seja, a direção de para onde
    o personagem está virado.*/
    private final int DOWN_FRAME = 6;
    private final int DOWN_FRAME_2 = 7;
    private final int UP_FRAME = 2;
    private final int RIGHT_FRAME = 4;
    private final int LEFT_FRAME = 0;
    
    public Boss(){}

    /*Construtor de Boss onde é passado pelo próprio construtor as posições x e y do Boss.*/
    public Boss(int coordenadaX, int coordenadaY){
        this.x = coordenadaX;
        this.y = coordenadaY;
    }

    /*Esse método é utilizado para criar um novo sprite no objeto Boss. É
     passado o arquivo em qual as imagens estão, o número de imagens que o arquivo
     contém, a largura e a altura das imagens separadamente para que o JavaPlay possa
     "entender" a organização dos seus sprites.*/
    public void setBoss(String arquivo, int numeroSprites, int largura, int altura){
        this.boss = new Sprite(arquivo, numeroSprites, largura, altura);
    }

    public Sprite getBoss(){
        return boss;
    }

    /*O método tem o mesmo sentido do setBoss, mas agora estamos colocando o
     HP(vida) do inimigo no objeto.*/
    public void setHp(String arquivo, int numeroSprites, int largura, int altura){
        this.hp = new Sprite(arquivo, numeroSprites, largura, altura);
    }

    public Sprite getHp(){
        return hp;
    }

    public void setFrameAtual(int FrameAtual){
        this.FrameAtual = FrameAtual;
    }

    public void setFrameAtualHp(int FrameAtualHp){
        this.FrameAtualHp = FrameAtualHp;
    }

    public int getVidas(){
        return vidas;
    }

    public void setVidas(int vidas){
        this.vidas = vidas;
    }

    public void step(long timeElapsed) {
        /*Aqui, em cada ciclo do jogo, o método step da classe Boss é chamado, e em
         seguida ele chama o método movimentoBoss, isso faz com o o Boss se mova sem
         parar pela tela enquanto o jogo roda.(Para saber como o Boss se move veja o
         método movimentoBoss))*/
        movimentoBoss();
    }

    /*Método utilizado para preencher(desenhar) a tela do jogo com o Boss, seu
     HP(vida), e seu nome.*/
    public void draw(Graphics g) {
        boss.setCurrAnimFrame(FrameAtual);
        boss.draw(g, x, y);
        hp.setCurrAnimFrame(FrameAtualHp);
        hp.draw(g, 325, 560);
        g.setColor(Color.red);
        g.drawString("FEIJAN: ", 260, 603);
        g.setColor(Color.white);
    }

    /*Esse método faz com que o personagem Boss se mova pelo mapa. O Boss tem uma particularidade
     em seu movimento, ele persegue o player pelo mapa a medida que o player se move.
     Optamos por fazer isso dividindo a tela em quadrantes onde o player é a origem do plano. Logo
     a cada passagem o Boss irá verificar em que quadrante ele mesmo se encontra, após feito isso
     ele dará um passo em direção a origem do sistema, tanto no eixo x, como no eixo y.*/
    public void movimentoBoss(){
        
        i++;/*Está variável é usada para fazer o efeito de movimento no boss, mudando o sprite
            a cada 5 passagens.*/

        if(this.x < TelaJogar.xPlayer && isAndarLivre() == true){
            this.x++;
            if((FrameAtual == RIGHT_FRAME)&&(i >= 15)){
                setFrameAtual(RIGHT_FRAME + 1);
                i = 0;
            }else{
                if(i >= 5){
                    setFrameAtual(RIGHT_FRAME);
                }
            }
        }
        if(this.x > TelaJogar.xPlayer && isAndarLivre() == true){
            this.x--;
            if((FrameAtual == LEFT_FRAME &&(i >= 15))){
                setFrameAtual(LEFT_FRAME + 1);
                i = 0;
            }else{
                if(i >= 5){
                    setFrameAtual(LEFT_FRAME);
                }
            }
        }
        if(this.y > TelaJogar.yPlayer && isAndarLivre() == true){
            this.y--;
            if((FrameAtual == UP_FRAME) &&(i >= 15)){
                setFrameAtual(UP_FRAME + 1);
                i = 0;
            }else{
                if(i >= 5){
                    setFrameAtual(UP_FRAME);
                }
            }
        }
        if(this.y < TelaJogar.yPlayer && this.x >= TelaJogar.xPlayer &&isAndarLivre() == true){
            this.y++;
            if((FrameAtual == DOWN_FRAME)&&(i >= 15)){
                setFrameAtual(DOWN_FRAME + 1);
                i = 0;
            }else{
                if(i >= 5){
                    setFrameAtual(DOWN_FRAME);
                }
            }
        }else{
            if(this.y < TelaJogar.yPlayer && this.x <= TelaJogar.xPlayer &&isAndarLivre() == true){
                this.y++;
                if((FrameAtual == DOWN_FRAME_2)&&(i >= 15)){
                    setFrameAtual(DOWN_FRAME_2 + 1);
                    i = 0;
                }else{
                    if(i >= 5){
                        setFrameAtual(DOWN_FRAME_2);
                    }
                }
            }
        }
        //FIM DA IMPLEMENTAÇÃO DO MOVIMENTO SEGUINDO O PLAYER
    }

    /*Esse método foi feito para que assim que o jogador chegue ao nível 5, o Boss se randomize
     em algum canto da tela, para logo após começar a perseguissão.*/
    public void posicaoBoss(){
        int randomico = (int)(Math.random()*40);
        if((randomico <= 40) && (randomico >= 31)){
                //Boss no eixo X e começa na esquerda
                setFrameAtual(RIGHT_FRAME);
                this.y = (int)((Math.random()*453) + 50);
                this.x = NumerosControle.BORDA_ESQUERDA;
                bossEixo = RIGHT_FRAME;
            }else{
                if((randomico <= 30) && (randomico >= 21)){
                    //Boss no eixo x e começa na direita
                    setFrameAtual(LEFT_FRAME);
                    this.y = (int)((Math.random()*453) + 50);
                    this.x = NumerosControle.BORDA_DIREITA - NumerosControle.LARGURA_INIMIGO;
                    bossEixo = LEFT_FRAME;
                }else{
                    if((randomico <= 20) && (randomico >= 11)){
                        //Boss no eixo y e começa em cima
                        setFrameAtual(DOWN_FRAME);
                        this.x = (int)((Math.random()*656) + 50);
                        this.y = NumerosControle.BORDA_SUPERIOR;
                        bossEixo = DOWN_FRAME;
                    }else{
                        if((randomico <= 10) && (randomico >= 1)){
                            //Boss no eixo y e começa em baixo
                            setFrameAtual(UP_FRAME);
                            this.x = (int)((Math.random()*656) + 50);
                            this.y = NumerosControle.BORDA_INFERIOR - NumerosControle.ALTURA_INIMIGO;
                            bossEixo = UP_FRAME;
                        }
                    }
                }
            }
    }

    /*Método para analisar se é permitido o Boss andar pela tela.*/
    public boolean isAndarLivre() {
        return andarLivre;
    }

    /*Método para dizer se é permitido o Boss andar na tela ou não.*/
    public void setAndarLivre(boolean andarLivre) {
        this.andarLivre = andarLivre;
    }

}
