/*
Desenvolvedores: Evandro Mendonça - evandro.user@gmail.com
                 Mateus Pelegrino - mateuspelegrino@gmail.com
Colaboração: Esteban Clua, Marcelo Zamith, Calebe Tonassi, Elaine Cifoni.
*/

/*COMENTÁRIO DA CLASSE
    Esta classe é responsável pelos inimigos no jogo. A classe gerência
 tudo que está relacionado aos inimigos, como movimento, sprites por exemplo.
 Os inimigos são randomizados pelos cantos da tela, e percorrem a tela até a outra
 extremidade em linha reta horizontal ou vertical, depois sao randomizados novamente.
*/

package aliensfront;

import java.awt.Graphics;
/*Importações de classes do JavaPlay, a primeira é usada para todo objeto do jogo,
 como personagens ou qualquer coisa que tenha um papel interativo no jogo. A segunda
 é usada para criar um Sprite, ou seja, um conjunto de desenhos que compõe o
 personagem ou objeto.*/
import javaPlay.GameObject;
import javaPlay.Sprite;

public class Enemy extends GameObject {

    private Sprite enemy;
    private int FrameAtual;
    private int inimigoEixo;
    /*Verifica se o Inimigo tem a permisão para andar livremente pela tela.*/
    private boolean andarLivre = true;
    /*Verifica se o Inimigo explodiu.*/
    private boolean explodiu = false;
    /*Declara a posição de orientação dos sprites, ou seja, a direção de para onde
    o personagem está virado.*/
    private final int DOWN_FRAME = 0;
    private final int UP_FRAME = 1;
    private final int RIGHT_FRAME = 2;
    private final int LEFT_FRAME = 3;

    public Enemy(){}

    /*Construtor do inimigo onde é passado pelo próprio construtor as posições x e y dos inimigos.*/
    public Enemy(int coordenadaX, int coordenadaY){
        this.x = coordenadaX;
        this.y = coordenadaY;
    }

    /*Esse método é utilizado para criar um novo sprite no objeto inimigo. É
     passado o arquivo em qual as imagens estão, o número de imagens que o arquivo
     contém e a altura e largura das imagens separadamente para que o JapaPlay possa
     "entender" a organização dos seus sprites.*/
    public void setEnemy(String arquivo, int numeroSprites, int largura, int altura){
        this.enemy = new Sprite(arquivo, numeroSprites, largura, altura);
    }

    public Sprite getEnemy(){
       return enemy;
   }

    public void setFrameAtual(int FrameAtual){
        this.FrameAtual = FrameAtual;
    }

    public void step(long timeElapsed) {
        /*Aqui, em cada ciclo do jogo, o método step da classe Emeny é chamado, e em
         seguida ele verifica se o inimigo morreu, caso afirmativo executa os métodos
         posiçãoInimigo e marca Explodiu como false, e continua a movimentação do inimigo.
         (Para saber como tudo funciona veja os métodos posiçãoInimigo, setExplodiu e
         movimentoInimigo)*/
        if(getExplodiu() == true){
            posicaoInimigo();
            setExplodiu(false);
        }
        movimentoInimigo();
    }

    /*Método utilizado para preencher(desenhar) a tela do jogo com os respectivos
     inimigos de cada fase.*/
    public void draw(Graphics g){
        enemy.setCurrAnimFrame(FrameAtual);
        enemy.draw(g, x, y);
    }

    /*Nesse método é feito a randomização da posição do inimigo na hora que ele
     "nasce" em algum canto da tela. O método escolha a posição, a direção e o
     sentido que o inimigo irá se mover.*/
    public void posicaoInimigo(){
        int randomico = (int)(Math.random()*40);
        if((randomico <= 40) && (randomico >= 31)){
                //inimigo anda no eixo X e começa na esquerda
                setFrameAtual(RIGHT_FRAME);
                this.y = (int)((Math.random()*453) + 50);
                this.x = NumerosControle.BORDA_ESQUERDA;
                inimigoEixo = RIGHT_FRAME;
            }else{
                if((randomico <= 30) && (randomico >= 21)){
                    //inimigo anda no eixo x e começa na direita
                    setFrameAtual(LEFT_FRAME);
                    this.y = (int)((Math.random()*453) + 50);
                    this.x = NumerosControle.BORDA_DIREITA - NumerosControle.LARGURA_INIMIGO;
                    inimigoEixo = LEFT_FRAME;
                }else{
                    if((randomico <= 20) && (randomico >= 11)){
                        //inimigo anda no eixo y e começa em cima
                        setFrameAtual(DOWN_FRAME);
                        this.x = (int)((Math.random()*656) + 50);
                        this.y = NumerosControle.BORDA_SUPERIOR;
                        inimigoEixo = DOWN_FRAME;
                    }else{
                        if((randomico <= 10) && (randomico >= 1)){
                            //inimigo anda no eixo y e começa em baixo
                            setFrameAtual(UP_FRAME);
                            this.x = (int)((Math.random()*656) + 50);
                            this.y = NumerosControle.BORDA_INFERIOR - NumerosControle.ALTURA_INIMIGO;
                            inimigoEixo = UP_FRAME;
                        }
                    }
                }
            }
    }

    /*Esse método é responsável pela movimentação do inimigo logo após a randomização
     feita acima. O inimigo continuará no mesmo eixo, mesma direção e sentido, e irá
     percorrer a tela de jogo até a próxima extremidade. Quando eles acabarem sua
     trajetória, serão randomizados novamente pelo método acima.*/
    public void movimentoInimigo(){
        if((inimigoEixo == RIGHT_FRAME) && (isAndarLivre() == true)){
            this.x += 2;
            if(this.x > NumerosControle.BORDA_DIREITA - NumerosControle.LARGURA_INIMIGO){
                posicaoInimigo();
            }
        }
        if((inimigoEixo == LEFT_FRAME) && (isAndarLivre() == true)){
            this.x -= 2;
            if(this.x < NumerosControle.BORDA_ESQUERDA){
                posicaoInimigo();
            }
        }
        if((inimigoEixo == DOWN_FRAME) && (isAndarLivre() == true)){
            this.y += 2;
            if((this.y > NumerosControle.BORDA_INFERIOR - NumerosControle.ALTURA_INIMIGO)||(this.x<NumerosControle.BORDA_ESQUERDA)){
                posicaoInimigo();
            }
        }
        if((inimigoEixo == UP_FRAME) && (isAndarLivre() == true)){
            this.y -= 2;
            if(this.y < NumerosControle.BORDA_SUPERIOR){
                posicaoInimigo();
            }
        }
    }

    /*Método responável para dizer se o inimigo ode ou não andar.*/
    public void pararInimigo(){
        setAndarLivre(false);
    }

    public int getEixo(){
        return inimigoEixo;
    }

    public boolean getExplodiu() {
        return explodiu;
    }

    public void setExplodiu(boolean explodiu) {
        this.explodiu = explodiu;
    }

    /*Método para analisar se é permitido o inimigo andar pela tela.*/
    public boolean isAndarLivre() {
        return andarLivre;
    }

    /*Método para dizer se é permitido o inimigo andar na tela ou não.*/
    public void setAndarLivre(boolean andarLivre) {
        this.andarLivre = andarLivre;
    }
}