/*
Desenvolvedores: Evandro Mendonça - evandro.user@gmail.com
                 Mateus Pelegrino - mateuspelegrino@gmail.com
Colaboração: Esteban Clua, Marcelo Zamith, Calebe Tonassi, Elaine Cifoni.
*/

/* COMENTÁRIO DA CLASSE
	Esta classe é a maior classe do jogo, pois esta tela carrega todos os
	steps dos personagens, verifica a fase, verifica a colisão, verifica 
	o ataque do player, avança a fase, desenha os componentes na tela do
	jogo, inicializa os inimigos, entre outros.
*/  

package aliensfront;

import java.awt.Graphics;
import java.awt.Point;
/* Importações de classes do JavaPlay, a primeira é a classe que permite
 a execução lógica do jogo, dividindo a execução em tempos e permitindo o 
 programador cuidar da lógica sob isso. A segunda é a classe principal do 
 JavaPlay que permite visibilidade de outras classes da engine.*/
import javaPlay.GameStateController;
import javaPlay.GameEngine;

class TelaJogar  implements GameStateController{
    
    private Fase nivel = new Fase(); 
    private Player player = new Player(400,300); 
    private TelaPerder perde = new TelaPerder(); 
    private Enemy[] enemy; 
    private Boss boss;
    private Auria auria;
    private int vidasPerdidasBoss = 0;
    private boolean delayLevel1 = true;
    private boolean colidiuAuria = false;

    /*Coordenadas do player e do boss*/
    public static int xPlayer;
    public static int yPlayer;
    public static int xBoss;
    public static int yBoss;
    
    public TelaJogar() {
        
    }
	
    /*No método load fica tudo que será carregado quando for o método for chamado*/
    public void load() {
        nivel.setNivelFase(1);//Seta o nivel 1
        player.setPlayer("MARCUS_SPRITE.png", 5, NumerosControle.LARGURA_PLAYER, NumerosControle.ALTURA_PLAYER);//Inicializa o player
        criaInimigos(nivel.getNumeroInimigos());//Inicializa os inimigos
    }

    public void unload() {

    }
	
	/*Método que inicia o jogo com a tela delay fase, o player no centro da tela,
	cria os inimigos e seta a quantidade de vida que o boss perdeu para 0*/
    public void start() {
        vidasPerdidasBoss = 0;
        if((nivel.getNivelFase() == 1) &&(delayLevel1 == true)){
            GameEngine.getInstance().setNextGameStateController(NumerosControle.DELAY_FASE);
            player.x = 400;
            player.y = 300;
            criaInimigos(nivel.getNumeroInimigos());
            delayLevel1 = false;
        }
    }
	
	/*Método que ocorrerá a cada ciclo do jogo, ou seja, a cada passagem ajustará 
	a velocidade do jogo, neste caso, diminuindo a velocidade. Ainda no Step, os
	cria inimigos são criados, sendo que cada nivel tem uma quantidade de inimigos 
	estipulada. E outros métodos são chamados*/
    public void step(long timeElapsed) {

        try { 
        Thread.sleep(25);
        } catch (Exception e) {
            e.printStackTrace();
        }
        
        avancaNivel();
        perde.step(timeElapsed);
        nivel.step(timeElapsed);

        for(int i = 0; i < nivel.getNumeroInimigos(); i++){
            enemy[i].step(timeElapsed);
        }

        player.step(timeElapsed);

        if(nivel.getNivelFase() == 5){//Se for o nivel do boss
            xPlayer = player.x;
            yPlayer = player.y;
            xBoss = boss.x;
            yBoss = boss.y;
            boss.step(timeElapsed);
        }

        playerColidiu();
        perdeuJogo();

    }//Fim do método Step
	
	/*Método draw é usado para preencher(desenhar) a tela com o nível, o player,
	os inimigos, e se for do nível do Boss, ele é desenhado e se ele morrer, a Auria é desenhada.*/
    public void draw(Graphics g) {
        nivel.draw(g);
        player.draw(g);
        for(int i = 0; i < nivel.getNumeroInimigos(); i++){
            enemy[i].draw(g);
        }

        if(nivel.getNivelFase() == 5){//Se for o nivel do boss
            boss.draw(g);
            if(boss.getVidas() == 0){
                auria.draw(g);
            }
        }
        
    }
    public void stop() {

    }

    public void playerColidiu(){
		/*Pega as coordenadas do player*/
        Point playerMin = new Point(player.x, player.y);
        Point playerMax = new Point(player.x + NumerosControle.LARGURA_PLAYER, player.y + NumerosControle.ALTURA_PLAYER);

        for(int i = 0; i < nivel.getNumeroInimigos(); i++){
			
			/*Pega as coordenadas dos inimigos, com um pequeno espaço para dentro*/
            Point enemyMin = new Point(enemy[i].x + 7, enemy[i].y + 2);
            Point enemyMax = new Point(enemy[i].x + NumerosControle.LARGURA_INIMIGO - 6, enemy[i].y + NumerosControle.ALTURA_INIMIGO - 4);

            /*Verifica se o player colidiu com o inimigo sem atacá-lo*/
            if(player.playerColidiu(playerMin, playerMax, enemyMin, enemyMax) && !player.ataquePlayer()){

                enemy[i].posicaoInimigo();
                player.setVidas(player.getVidas() - 1 );
                player.x = 400;
                player.y = 300;
                player.setFrameAtual(0);

            }else{
				
				/*Pega as coodernadas dos inimigos com um espaço*/
                enemyMin = new Point(enemy[i].x - 10, enemy[i].y - 10);
                enemyMax = new Point(enemy[i].x + NumerosControle.LARGURA_INIMIGO + 10, enemy[i].y + NumerosControle.ALTURA_INIMIGO + 10);

                /*Verifica se o player colidiu com o inimigo atacando-o alinhado*/
                if((player.playerColidiu(playerMin, playerMax, enemyMin, enemyMax) && player.ataquePlayer() && playerAlinhadoInimigo(i)) == true ){
                    enemy[i].setFrameAtual(4);
                    enemy[i].setExplodiu(true);
                    /*Desenha o inimigo explodindo antes de setar a posição novamente*/
                     player.setNumeroPontos(player.getNumeroPontos() + 10);

                }else{
				/*Se o player colidiu com o inimigo atacando-o desalinhado*/
                    if((player.playerColidiu(playerMin, playerMax, enemyMin, enemyMax) && player.ataquePlayer()) == true && (playerAlinhadoInimigo(i)) == false){
                        enemy[i].posicaoInimigo();
                        player.setVidas(player.getVidas() - 1 );
                        player.x = 400;
                        player.y = 300;
                        player.setFrameAtual(0);
                    }

                }
            }
        }

        /*Verifica se colidiu com o boss*/
        if(nivel.getNivelFase() == 5){
            Point bossMin = new Point(boss.x , boss.y);
            Point bossMax = new Point(boss.x + NumerosControle.LARGURA_BOSS, boss.y + NumerosControle.ALTURA_BOSS);
            if(player.playerColidiu(playerMin, playerMax, bossMin, bossMax) && !player.ataquePlayer()){//Player não atacou e colidiu
                player.setVidas(player.getVidas() - 1 );
                player.x = 400;
                player.y = 300;
                boss.posicaoBoss();
            }else{/*Player atacou e colidiu*/
                bossMin = new Point(boss.x - 10, boss.y - 10);
                bossMax = new Point(boss.x + NumerosControle.LARGURA_BOSS + 10, boss.y + NumerosControle.ALTURA_BOSS + 10);
                if(player.playerColidiu(playerMin, playerMax, bossMin, bossMax) && player.ataquePlayer()){
                    boss.posicaoBoss();
                    boss.setVidas(boss.getVidas() - 1);
                    vidasPerdidasBoss++;
                    boss.setFrameAtualHp(vidasPerdidasBoss);
                }
            }

            /*Verifica se o boss morreu, caso verdade, ele move todos os
			inimigos e o boss para uma posição e os impede de andar*/
            if(boss.getVidas() == 0){
                for(int i = 0; i < nivel.getNumeroInimigos(); i++){
                   enemy[i].x = 2000;
                   enemy[i].y = 2000;
                   enemy[i].setAndarLivre(false);
                }

                boss.x = 2000;
                boss.y = 2000;
                boss.setAndarLivre(false);
				
				/*Inicializa a Auria, com seu sprite e sua posição*/
                auria = new Auria("AURIA_SPRITE.png", 1, NumerosControle.LARGURA_AURIA, NumerosControle.ALTURA_AURIA);
                auria.x = 450;
                auria.y = 400;
				
				/*Verifica se o player colidiu com a auria*/
                Point auriaMin = new Point(auria.x , auria.y);
                Point auriaMax = new Point(auria.x + NumerosControle.LARGURA_AURIA, auria.y + NumerosControle.ALTURA_AURIA);
                if(player.playerColidiu(playerMin, playerMax, auriaMin, auriaMax)){
                        colidiuAuria = true;
                }
            }
        }
    }
	
	/*Este método verifica primeiro se o tempo é igual a zero, caso positivo 
	ele verifica o nível que o jogador se encontra. E se for maior ou igual que
	a pontuação necessária para avançar de fase, o delay fase é chamado, o player
	é movido para o centro da tela e a quantidade de inimigo do level é criada.
	Senão, a tela perdeu é chamada. Se o nível for o 5, caso o tempo acabe e o 
	número de vidas do boss for maior que 0, a tela perdeu é chamada. Se o boss
	for morto e o player colidir com a Auria, a tela ganhou é chamada
	*/
    public void avancaNivel(){
        if(nivel.getTime() == 0.0f){
            switch (nivel.getNivelFase()){
                case 1:
                    if(player.getNumeroPontos() >= 50){
                        nivel.setNivelFase(2);
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.DELAY_FASE);
                        player.x = 400;
                        player.y = 300;
                        criaInimigos(nivel.getNumeroInimigos());
                    }else{
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_PERDEU);
                    }
                    break;
                case 2:
                    if(player.getNumeroPontos() >= 120){
                        nivel.setNivelFase(3);
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.DELAY_FASE);
                        player.x = 400;
                        player.y = 300;
                        criaInimigos(nivel.getNumeroInimigos());
                    }else{
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_PERDEU);
                    }
                    break;
                case 3:
                    if(player.getNumeroPontos() >= 250){
                        nivel.setNivelFase(4);
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.DELAY_FASE);
                        player.x = 400;
                        player.y = 300;
                        criaInimigos(nivel.getNumeroInimigos());
                    }else{
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_PERDEU);
                    }
                    break;
                case 4:
                    if(player.getNumeroPontos() >= 350){
                        nivel.setNivelFase(5);
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.DELAY_FASE);
                        player.x = 400;
                        player.y = 300;
                        criaInimigos(nivel.getNumeroInimigos());
                        criaBoss();
                    }else{
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_PERDEU);
                    }
                    break;
                case 5:
                    if(boss.getVidas() > 0){
                        GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_PERDEU);
                    }else{
                        if(player.getNumeroPontos() >= 350 && boss.getVidas() == 0){
                            GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_GANHOU);
                        }
                    }
            }
        }

        if(nivel.getNivelFase() == 5){
            if(boss.getVidas() == 0){
                if(colidiuAuria == true){
                    GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_GANHOU);
                }
            }
        }
    }
	
	/*Verifica se o player morreu, ou seja, se sua vida é 0, caso verdade, a tela perdeu é chamada*/
    public void perdeuJogo(){
        if(player.getVidas() <= 0){
            GameEngine.getInstance().setNextGameStateController(NumerosControle.TELA_PERDEU);
        }
    }
	
	/*Cria a quantidade de inimigos estipulada para cada fase*/
    private void criaInimigos(int numeroInimigos) {
        enemy = new Enemy[numeroInimigos];
        for(int i = 0; i < numeroInimigos ;i++){
            enemy[i] = new Enemy();
            enemy[i].setEnemy("ALIEN_SPRITE.png", 11, NumerosControle.LARGURA_INIMIGO, NumerosControle.ALTURA_INIMIGO);
            enemy[i].posicaoInimigo();
        }
    }
	
	/*Método para criar e setar os sprites referentes ao Boss*/
    public void criaBoss(){
        boss = new Boss();
        boss.setBoss("BOSS_SPRITE.png", 8, NumerosControle.LARGURA_BOSS, NumerosControle.ALTURA_BOSS);
        boss.posicaoBoss();
        boss.setHp("BOSS_HP.png", 7, NumerosControle.LARGURA_HP, NumerosControle.ALTURA_HP);
    }
	
	/*Verifica se o player está na mesma posição do inimigo*/
    public boolean playerAlinhadoInimigo(int contador){

        if ((player.getEixo() == 0) && ((enemy[contador].getEixo() == 1)||(enemy[contador].getEixo() == 2)||(enemy[contador].getEixo() == 3))){
            return true;
        }
        if ((player.getEixo() == 1) && ((enemy[contador].getEixo() == 0)||(enemy[contador].getEixo() == 2)||(enemy[contador].getEixo() == 3))){
            return true;
        }
        if ((player.getEixo() == 2) && ((enemy[contador].getEixo() == 3)||(enemy[contador].getEixo() == 0)||(enemy[contador].getEixo() == 1))){
            return true;
        }
        if ((player.getEixo() == 3) && ((enemy[contador].getEixo() == 2)||(enemy[contador].getEixo() == 0)||(enemy[contador].getEixo() == 1))){
            return true;
        }
        return false;
    
   
	}
}